package org.ogce.xregistry.client;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.security.GeneralSecurityException;
import java.security.cert.X509Certificate;

import org.globus.gsi.ptls.PureTLSContext;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ogce.xregistry.utils.XRegistryClientException;
import org.xmlpull.v1.builder.XmlBuilderException;
import org.xmlpull.v1.builder.XmlElement;

import xregistry.generated.IXregistryPortType;
import xsul.XmlConstants;
import xsul.XsulException;
import xsul.invoker.gsi.GsiInvoker;
import xsul.invoker.puretls.PuretlsInvoker;
import xsul.invoker.soap_over_http.SoapHttpDynamicInfosetInvoker;
import xsul.wsdl.WsdlDefinitions;
import xsul.wsif.WSIFException;
import xsul.wsif.WSIFService;
import xsul.wsif.WSIFServiceFactory;
import xsul.wsif_xsul_soap_gsi.Provider;
import xsul.wsif_xsul_soap_http.XsulSoapPort;
import xsul.xwsif_runtime.WSIFClient;
import xsul.xwsif_runtime.XmlBeansWSIFRuntime;
import COM.claymoresystems.sslg.SSLPolicyInt;

public class XRegistryConnection {
	
	/**
	 * Create a Non-secure XRegistry Connection Stub
	 * @param nonSecureXRegistryUrl
	 * @return xregistryStub
	 * @throws XRegistryClientException
	 */
	protected static IXregistryPortType createNonSecureServiceStub(String nonSecureXRegistryUrl) throws XRegistryClientException {
		
		WSIFClient client = XmlBeansWSIFRuntime.newClient(nonSecureXRegistryUrl);
		return (IXregistryPortType)client.generateDynamicStub(IXregistryPortType.class);
		
	}
	
	/**
	 * Create a GSI Secure XRegistry Connection Stub
	 * @param credential
	 * @param certs
	 * @param xregistryServiceWsdlUrl
	 * @return
	 * @throws XRegistryClientException
	 */
	protected static IXregistryPortType createGSISecureServiceStub(GSSCredential gssCredential, X509Certificate[] trustedCerts,
			String xregistryServiceWsdlUrl) throws XRegistryClientException {
		try {
			SoapHttpDynamicInfosetInvoker invoker = new GsiInvoker(gssCredential, trustedCerts);
			String wsdlAsStr = invoker.invokeHttpGet(xregistryServiceWsdlUrl);
			XmlElement el = XmlConstants.BUILDER.parseFragmentFromReader(new StringReader(wsdlAsStr));
			WsdlDefinitions def = new WsdlDefinitions(el);

			WSIFServiceFactory wsf = WSIFServiceFactory.newInstance();
			WSIFService serv = wsf.getService(def);
			serv.addLocalProvider(new Provider(invoker));
			
			WSIFClient client = XmlBeansWSIFRuntime.getDefault().newClientFor(serv.getPort());
			((XsulSoapPort) client.getPort()).setInvoker(invoker);
			return (IXregistryPortType)client.generateDynamicStub(IXregistryPortType.class);
			
		} catch (WSIFException e) {
			throw new XRegistryClientException(e);
		} catch (XsulException e) {
			throw new XRegistryClientException(e);
		} catch (XmlBuilderException e) {
			throw new XRegistryClientException(e);
		}
	}
	
	/**
	 * Create a SSL Secure XRegistry Connection Stub
	 * @param hostkeyPath
	 * @param trustedCerts
	 * @param xregistryServiceWsdlUrl
	 * @return
	 * @throws XRegistryClientException
	 * @throws GeneralSecurityException 
	 */
	protected static IXregistryPortType createSSLSecureServiceStub(String hostkeyPath, X509Certificate[] trustedCerts,
			String xregistryServiceWsdlUrl) throws XRegistryClientException, GeneralSecurityException,
			FileNotFoundException, IOException{
		try {
			if (hostkeyPath == null || hostkeyPath == "") {
				throw new XRegistryClientException("The host key file path is null or empty");
			}
			PureTLSContext ctx = new PureTLSContext();
			ctx.setTrustedCertificates(trustedCerts);
			ctx.loadEAYKeyFile(hostkeyPath, "");
			SSLPolicyInt policy = new SSLPolicyInt();
			policy.requireClientAuth(true);
			policy.setAcceptNoClientCert(true);
			ctx.setPolicy(policy);
			SoapHttpDynamicInfosetInvoker invoker = new PuretlsInvoker(ctx);
			String wsdlAsStr = invoker.invokeHttpGet(xregistryServiceWsdlUrl);
			XmlElement el = XmlConstants.BUILDER.parseFragmentFromReader(new StringReader(wsdlAsStr));
			WsdlDefinitions def = new WsdlDefinitions(el);

			WSIFServiceFactory wsf = WSIFServiceFactory.newInstance();
			WSIFService serv = wsf.getService(def);
			serv.addLocalProvider(new Provider(invoker));
			
			WSIFClient client = XmlBeansWSIFRuntime.getDefault().newClientFor(serv.getPort());
			((XsulSoapPort) client.getPort()).setInvoker(invoker);
			return (IXregistryPortType)client.generateDynamicStub(IXregistryPortType.class);
			
		} catch (WSIFException e) {
			throw new XRegistryClientException(e);
		} catch (XsulException e) {
			throw new XRegistryClientException(e);
		} catch (IOException e) {
			throw new XRegistryClientException(e);
		} catch (GeneralSecurityException e) {
			throw new XRegistryClientException(e);
		} catch (XmlBuilderException e) {
			throw new XRegistryClientException(e);
		}
	}
}
